var listArray = new Array();

function bindListEvents(listName, totalRows, phpFile, hiddenRowClass) {
    var args = new Array();
    for (i = 4, j = 0; i < bindListEvents.arguments.length; i++, j++) {
        args[j] = bindListEvents.arguments[i];
    }
    var list = $('#' + listName);
    var listObject = new List(listName, totalRows, phpFile, hiddenRowClass, args, listArray);
    listArray.push(listObject);
    $(list).resize(function(event) {
        listObject.checkTimer(event);
    });
    $(list).scroll(function(event) {
        listObject.checkTimer(event);
    });
    $(list).ready(function(event) {
        listObject.checkTimer(event);
    });
}

function getListByName(listName) {
    for (i = 0; i < listArray.length; ++i) {
        if (listArray[i].name == listName) {
            return(listArray[i]);
        }
    }
    return null;
}

function updateList(listName) {
    console.info("updating list " + listName);
    list = getListByName(listName);
    if (list) 
        list.checkItemsInView();
    else
        console.error("list " + listName + " not found");
}

// "class" list
function List(name, totalRows, phpFile, hiddenRowClass, args, lists) {
    this.name = name;
    this.totalRows = totalRows;
    this.phpFile = phpFile;
    this.lastIndex = 0;
    this.currentSelectedIndex = -1;
    this.scheduledSelectedIndex = 0;
    this.curScrollTop = 0;
    this.workingEvent = null;
    this.event = null;
    this.blocking = false;
    this.preLoadOffset = 20;                // items which are rendered before and after the viewable area to give a better scroll fealing
    this.hiddenRowClass = hiddenRowClass;
    this.args = args;
    this.lists = lists;
    this.rowsInView = 0;
    
    this.checkTimer = function(event) {
        this.event = event;
        if (!this.blocking) {
            this.blocking = true;
            this.workingEvent = event;
            window.setTimeout(this.checkItemsInView.bind(this),
                10
                )
        }
    }

    this.replaceRows = function(data, index) {
        listArray = this.lists;
        // hide the last visible rows
        if (this.lastIndex < index) {
            for (i = this.lastIndex; i < index && i < (this.lastIndex + this.rowsInView) && i < this.totalRows; i++) {
                this.hideRow(i);
            }
        }
        else if (this.lastIndex > index) {
            var diff = this.lastIndex - index;
            var start = 0;
            if (diff < this.rowsInView) {
                start = this.lastIndex + (this.rowsInView - diff);
            }
            else {
                start = this.lastIndex;
                diff = this.rowsInView;
            }
            for (i = start; i < (this.lastIndex + this.rowsInView) && i < this.totalRows; i++) {
                this.hideRow(i);
            }
        }
        this.lastIndex = index;
        // get the first row item
        var rowItem = $('#' + this.name + index)[0];
        var i = index + 1;
        // remove all further row items in view
        for (; i < (index + this.rowsInView); i++) {
            $('#' + this.name + i).remove();
        }
        // and replace them
        $(rowItem).replaceWith(data);
        // we have no current item, so make sure it will be selected later on
        console.info("currentSelectedIndex " + this.currentSelectedIndex + " scheduledSelectedIndex " + this.scheduledSelectedIndex);
        if (this.currentSelectedIndex >= 0) this.scheduledSelectedIndex = this.currentSelectedIndex;
        // this.currentSelectedIndex = -2; 
        this.blocking = false;
        if (this.workingEvent != this.event) {
            // there is a newer event since the last one, so execute this one too
            this.checkTimer(this.event);
        }
        
        // Trigger change Event on list
        $('#' + this.name).trigger("change");        
    }

    this.checkItemsInView = function() {
        var rowHeight = $('#' + this.name + 0).height();    // determine row height
        var list = $('#' + this.name)[0];                   // get list div
        this.rowsInView = Math.ceil($(list).height() / rowHeight) + (this.preLoadOffset * 2);    // calc rows, which are visible and add a preload offset
        if (this.workingEvent.type == "scroll") {                                                // get the current scrolltop for scroll events
            this.curScrollTop = $(list).scrollTop();
        }
        //alert(this.curScrollTop + " " + rowHeight + " " + rowsInView);
        var index = Math.floor(this.curScrollTop / rowHeight);
        var i, j;
        var rowItem;
        while (!this.inView(index)) {    // a hack for the IE to determine the real first in view index
            index++;
        }
        if (index > this.preLoadOffset) {
            index -= this.preLoadOffset;
        }
        // build ids array, which will be posted to PHP
        var ids = Array();
        for (i = index, j = 0; i < (index + this.rowsInView) && i < this.totalRows; i++, j++) {
            rowItem = $('#' + this.name + i)[0];
            var rowId = $(rowItem).attr("name");
            ids[j] = rowId;
        }
        // alert('first id ' + ids[0] + ' #rows ' + this.totalRows + ' lastIndex ' + this.lastIndex);

        var listObject = this;
        // ajax request which gets rowsInView and replaces the dummy rows
        $.post(this.phpFile,
        {
            _method: "getRowsAsync", 
            _methodfield: this.name, 
            ids: ids, 
            firstrow: index, 
            args: this.args
        },
        function(data) {
            // this is called when all rows have been retrieved, so replace them on screen ...
            listObject.replaceRows(data, index);
            // ... and select the proper one (to begin with, we simply click the first item in list)
            var row = document.getElementById(listObject.name+listObject.scheduledSelectedIndex);
            if (row) userActionList(null, row, selectItemList);
            else {
                alert("index " + listObject.scheduledSelectedIndex + " out of range for " + listObject.name + " totalrows=" + listObject.totalRows);
            }
        }
        );
    }

    // this function replaces a real row with a hidden dummy row
    this.hideRow = function(index) {
        var rowItem = $('#' + this.name + index)[0];
        var rowId = $(rowItem).attr("name");
        $(rowItem).replaceWith("<tr class='" + this.hiddenRowClass + "' id='" + this.name + index + "' name='" + rowId + "' data-linenumber='" + index + "'><td><div>&nbsp;</div></td></tr>");
    }
    

    // this function determines if a row is in view or not
    this.inView = function(index) {
        var elem = $('#' + this.name + index)[0];
        if (!elem) return true;
        var vpH = getViewportHeight();
        var scrolltop = $('#' + this.name).offset().top;
        var top = $(elem).offset().top;
        var height = $(elem).height();
        if (scrolltop > (top + height) || scrolltop + vpH < top) {
            return false;
        }
        else {
            return true;
        }
    }
}

function getViewportHeight() {
    var height = window.innerHeight; // Safari, Opera
    var mode = document.compatMode;
    if ((mode || !$.support.boxModel)) { // IE, Gecko
        height = (mode == 'CSS1Compat') ?
        document.documentElement.clientHeight : // Standards
        document.body.clientHeight; // Quirks
    }
    return height;
}

/* this gives the possibility to add/remove license for an InputPageListField */

// retrieves index of a list name of format name@index
function getIndex(name) {
    re = RegExp("[^\@]\@([0-9]*)");
    match = re.exec(name);
    return match[1];
}

function listAddRow(listName, firstElementName) {
    var select = document.getElementsByName(firstElementName);
    var tr = select[0].parentNode.parentNode;
    var newTr = tr.cloneNode(true);
    var table = tr.parentNode;
    var index = table.childNodes.length;
    // change all name indices
    listReplaceIndex(newTr.childNodes, 0, index, true);
    //newTr.innerHTML = newTr.innerHTML.replace("removeLine('" + firstElementName + "', 0)", "removeLine('" + firstElementName + "', " + (index++).toString() + ")");
    table.appendChild(newTr);
    var listCount = document.getElementsByName("_" + listName + "_count");
    listCount[0].value = index + 1;
}

function listRemoveRow(listName, name) {
    var select = document.getElementsByName(name);
    var index = parseInt(getIndex(name));
    var tr = select[0].parentNode.parentNode.parentNode;
    var table = tr.parentNode;
    if (table.childNodes.length == 1) {
        listReplaceIndex(tr.childNodes, index, index, true);
        return;
    }
    var next = tr.nextSibling;
    table.removeChild(tr);
    while (next) {
        listReplaceIndex(next.childNodes, index + 1, index, false);
        next = next.nextSibling;
        index++;
    }

    var listCount = document.getElementsByName("_" + listName + "_count");
    listCount[0].value = index;
}

function listReplaceIndex(childNodes, indexBefore, indexAfter, defaultValues) {
    for (var i = 0; i < childNodes.length; i++) {
        if (childNodes[i].name) {
            childNodes[i].name = childNodes[i].name.replace("@" + indexBefore, "@" + indexAfter);
            if (defaultValues) {
                switch (childNodes[i].type) {
                    case "select-one":
                        childNodes[i].selectedIndex = 0;
                        break;
                    case "password":
                    case "text":
                        childNodes[i].value = "";
                        break;
                }
            }
        }
        if (childNodes[i].childNodes.length) {
            listReplaceIndex(childNodes[i].childNodes, indexBefore, indexAfter, defaultValues);
        }
    }
}

/*
 * event bubbling protection support
 */

        
function stopEventPropagationList(e) {
    if (!e) e = window.event;  // for non firefox
    if (e) { // if e not valid, then we have not been called by an event, so skip
        e.cancelBubble = true;
        if (e.stopPropagation) e.stopPropagation();
    }
}

/*
 * this function retrieves relevant date from the list and row and then calls a user defined function
 */
function userActionList(e, element, func) {
    // e is for event bubbling protection support to be able to call stopEventPropagationList
    // the element clicked
    var me = $(element);
    // the row we are on
    var row = ($(me).prop('nodeName') != 'tr') ? me.closest('tr') : me;
    var lino = row.data('linenumber');
    // the list
    var closestTable = row.closest('table');
    var rowParents = row.parents('table');
    var list = closestTable.parent();
    var listid = list.attr('id');
    var listobject = getListByName(listid);
    // the row key
    var key = row.data('key');
    var keyfield = list.data('keyfield');
    // the optional details field
    var detailsfieldname = list.data('detailsfield');
    var detailsfield = (detailsfieldname == '') ? null : $('#' + detailsfieldname);
    // the optional details form
    var detailsformname = row.data('detailsform');
    
    return(func(e, me, row, lino, list, listobject, keyfield, key, detailsfieldname, detailsfield, detailsformname));
}


// the default onlick method on an InputPageAsyncListField
// requires an asyncIdField (which will probably be of type InputPageHiddenIdField) and an 
// asyncDetailsField (which will probably be of type InputPageAsyncField) hooked to the InputPageAsyncListField
function selectItemList(e, me, row, lino, list, listobject, keyfield, key, detailsfieldname, detailsfield, detailsformname) {
    
    console.info("selecting lino=" + lino);
    // do not propagate event
    stopEventPropagationList(e);

    // highlight current one (and deselect others)
    // needs to be done anyway as "selected" class may got lost during reload
    list.find('tr').removeClass('selected');
    row.addClass('selected');

    // see if we are already selected
    if (lino == listobject.currentSelectedIndex) {
        console.info("already on this line");
        return false;
    }
    listobject.scheduledSelectedIndex = listobject.currentSelectedIndex = lino;

    // update details form (if there is one)
    if (detailsfieldname != '') {
        var dingy = new Object;
        dingy[keyfield] = key;
        console.info("reloading details form " + detailsformname);
        console.dir(dingy);
        detailsfield.load(detailsformname, dingy ); 
    }
    return false;    
}


/*
 * update an (probably async) form with ajax, update the correspondign list if any
 */
function updateDetailsFormList(postto, detailsform, detailsfield, asynclist, reloadlist) {
    // TODO obsolete??
    var serializedForm = $('#'+detailsform).serialize();
    console.info("updating details form " + detailsform + " with " + serializedForm);
    $.post(postto, 
        serializedForm, 
        function (data) { 
            if (detailsform != '' && detailsfield != '') {
                console.info("updating " + detailsfield);
                $('#'+detailsfield).html(data); 
            }
            if (asynclist != '' && reloadlist) updateList(asynclist); 
        }
        );
    return false;
}